"""'
Contains the Constants used by myansible modules
"""
NETORCA_STATES_ERRORED = "ERROR"
NETORCA_STATES_PENDING = "PENDING"
NETORCA_STATES_APPROVED = "APPROVED"
NETORCA_STATES_COMPLETED = "COMPLETED"
NETORCA_VALID_STATES = [
    NETORCA_STATES_ERRORED,
    NETORCA_STATES_PENDING,
    NETORCA_STATES_APPROVED,
    NETORCA_STATES_COMPLETED,
]
NETORCA_VALID_UPDATE_STATES = [NETORCA_STATES_ERRORED, NETORCA_STATES_APPROVED, NETORCA_STATES_COMPLETED]
NETORCA_VALID_CONTEXTS = ["owner", "consumer"]

FIELDS_URL = "url"
FIELDS_USER = "username"
FIELDS_API_KEY = "api_key"
FIELDS_PASS = "password"
FIELDS_STATE = "state"
FIELDS_SERVICE = "service_name"
FIELDS_DEPLOYED_ITEM = "deployed_item"
FIELDS_CONTEXT = "context"
FIELDS_OBJECT_ID = "uuid"
FIELDS_FILTERS = "filters"

VALID_FILTERS = [
    FIELDS_SERVICE,
]
